#!/usr/bin/env python
# -*- coding: utf-8 -*-

#from vector_dict.VectorDict import VectorDict as vd,convert_tree
from archery.bow import Hankyu as _dict
from archery.barrack import bowyer as convert_tree


from os import path, environ
from json import load
from datetime import datetime as dt
from matplotlib import dates
from optparse import OptionParser
from numpy import array

##################### WORKFLOW ######################
###### parsing
###### querying
###### plotting
#####################################################

################# PARSING CLI #################################################
parser = OptionParser()
_key = ['total_dl' ]
parser.add_option("-k", dest="key",
    help='''keys to plot in stored stats amongst av_dl, total_dl, min_dl','max_dl' ''' ,
    action = 'append' ,
)

parser.add_option("-f",'--from', dest="_from",
    help='min date from which to plot', default = ''
)

parser.add_option("-t",'--to', dest="_to",
    help='maximum date to which to plot',
    default = dt.now().date().isoformat()
)

parser.add_option("-o",'--output', dest="_dest",
    help="""
    filename of the output image. Don't create output directory if non existant.
If this is enabled output in TK backend is desactivated.
    """,
    action='store'
)
parser.add_option("-p",'--package', dest="_package",
    help='packages for wich to graph',
    action='append'
    )
(options, args) = parser.parse_args()

#################### QUERYING DATA ############################################



my_filter = lambda sequence : filter(
        lambda r :  options._to >= r['date'] > options._from ,sequence
    )
all_filter = my_filter
if options._package:
    all_filter = lambda sequence : filter(
        lambda x : x["name"] in options._package, my_filter(sequence)
    )

_key = options.key  or _key
save=path.join(environ["HOME"], ".pipy.stat.json" )
result = load(open(save))

res = reduce( _dict.__iadd__,
    map(
        lambda x : convert_tree(_dict,
            {
                x["name"] : dict(
                    date =[ dates.date2num(dt.strptime(x['date'],"%Y-%m-%d")) ],
                )
            }) + convert_tree(_dict,
                {  x['name'] :
                    dict( ( key, [  int(x[key]) ] ) for key in _key)
            }) + convert_tree(_dict, {
                x["name"] : {
                    'release' :  { x["last_release"] :
                    [ x['last_upload'] ] }
                    }}
            ),
            sorted(
                all_filter(
                    result
                ),
                key = lambda r : r["name"]
            )
    )
)

options._package = res.keys()

################ PLOTTING #####################################################
import matplotlib
matplotlib.use(options._dest and "Agg" or "TkAgg")

import matplotlib.pyplot as plt
from matplotlib.ticker import FuncFormatter
K = 1000
YMAX_BUG=80*K
YSIZE_PER_PLOT_INCH = 4

total_plot = len(res)
fig = plt.figure(figsize = ( 8 , YSIZE_PER_PLOT_INCH *total_plot ))
min_date, max_date = None,None
ax1 = None
ax = None

for cursor, (name, data) in enumerate(res.iteritems()):
    if cursor != 0 :
        ax = fig.add_subplot( total_plot , 1, cursor +1, sharex=ax1)
    else:
        ax = fig.add_subplot(total_plot , 1,1 )
        ax1= ax

    _plot = []
    for key in _key:
        min_date = min_date and \
            min( min(data["date"]), min_date) or \
            min(data["date"])
        max_date = max_date and \
            max( max(data["date"]), max_date) or \
            max(data["date"])

    ymax=0
    for key in _key:
        ymax = max(max(data[key]), ymax)
        _plot += [ ax.plot_date( array(data["date"]), array(data[key], int),  '.-' , label = key) ]
    if ymax > YMAX_BUG:
        ax.yaxis.set_major_formatter(FuncFormatter(
            lambda x, pos: ('%.1f')%(x*1e-3))
        )
        ax.set_ylabel('K (1E3)')

        
        

    ax.legend(  loc=2,)

    for label, date in data["release"].iteritems():
        ax.annotate( "release\n" + label,
            xy = (
                dates.date2num(dt.strptime(date[0][0:10],"%Y-%m-%d"  )),
                ax.get_ybound()[0] + 1
            ),
            horizontalalignment='center', verticalalignment='center',
            xytext = (
                 dates.date2num(dt.strptime(date[0][0:10],"%Y-%m-%d"  )),
                 sum(ax.get_ybound())  / 2
            ),
            arrowprops=dict(facecolor='black')
        )

    plt.xlim([ min_date, max_date])

    ax.set_title( "Statistic for %s" % (name) )

fig.autofmt_xdate()
plt.autoscale(axis = 'y')
plt.subplots_adjust( top =.9 , bottom=0.2, left=0.15,right=.9)

plt.draw()

if options._dest:
    plt.savefig( options._dest )
else:
    plt.show()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Calculates the total number of downloads that a particular PyPI package has
received across all versions tracked by PyPI
author : codekoala (not me)
License : ask codekoala
source : http://www.codekoala.com/blog/2010/pypi-download-stats/
Added a save method
"""

import argparse
from datetime import datetime
import locale
import sys
import xmlrpclib
from os import path, environ
from json import load
from time import sleep
import shutil

locale.setlocale(locale.LC_ALL, '')

class PyPIDownloadAggregator(object):

    def __init__(self, package_name, include_hidden=True, quiet=False ):
        self.package_name = package_name
        self.include_hidden = include_hidden
        self.proxy = xmlrpclib.Server('https://pypi.python.org/pypi')
        self._downloads = {}
        self.quiet = quiet

        self.first_upload = None
        self.first_upload_rel = None
        self.last_upload = None
        self.last_upload_rel = None

    @property
    def releases(self):
        """Retrieves the release number for each uploaded release"""

        result = self.proxy.package_releases(self.package_name, self.include_hidden)

        if len(result) == 0:
            # no matching package--search for possibles, and limit to 15 results
            results = self.proxy.search({
                'name': self.package_name,
                'description': self.package_name
            }, 'or')[:15]

            # make sure we only get unique package names
            matches = []
            for match in results:
                name = match['name']
                if name not in matches:
                    matches.append(name)

            # if only one package was found, return it
            if len(matches) == 1:
                self.package_name = matches[0]
                return self.releases

            error = """No such package found: %s

Possible matches include:
%s
""" % (self.package_name, '\n'.join('\t- %s' % n for n in matches))

            sys.exit(error)

        
        return result

    @property
    def downloads(self, force=False):
        """Calculate the total number of downloads for the package"""

        if len(self._downloads) == 0 or force:
            for release in self.releases:
                urls = self.proxy.release_urls(self.package_name, release)
                self._downloads[release] = 0
                for url in urls:
                    # upload times
                    uptime = datetime.strptime(url['upload_time'].value, "%Y%m%dT%H:%M:%S")
                    if self.first_upload is None or uptime < self.first_upload:
                        self.first_upload = uptime
                        self.first_upload_rel = release

                    if self.last_upload is None or uptime > self.last_upload:
                        self.last_upload = uptime
                        self.last_upload_rel = release

                    self._downloads[release] += url['downloads']

        return self._downloads

    def total(self):
        return sum(self.downloads.values())

    def average(self):
        return self.total() / len(self.downloads)

    def max(self):
        return max(self.downloads.values())

    def min(self):
        return min(self.downloads.values())

    def save(self):
        from os import path, environ
        from json import load, dump, dumps
        sep = str
        from time import mktime
        from datetime import date 
        save=path.join(environ["HOME"], ".pipy.stat.json"  )
        
        json = dict(
            date = "%s" % date.today(),
            name = self.package_name,
            first_upload = '%s'  % self.first_upload,
            first_release =  self.first_upload_rel,
            last_upload = '%s' % self.last_upload,
            last_release  = sep(self.last_upload_rel),
            nb_release = sep(len(self.releases)),
            max_dl = sep(self.max()),
            min_dl = sep(self.min()),
            av_dl = sep(self.average()),
            total_dl = sep(self.total()),
        )
        if not path.exists(save):
            with file(save,"w") as f:
                f.write("[]")
        
        with open(save) as stored:
            result = filter( 
                lambda r : r["name"] != self.package_name or 
                    r["date"] != str(date.today()),
                load(stored)
            )
            result += [  json ]
        dump(result,open("%s.bak" % save ,"w"))
        try:
            ## move are atomic operations less dangerous than previous
            shutil.move(save, "%s.old" % save)
            shutil.move("%s.bak" % save,save)
        except Exception as e:
            print("something weired occured")
            print("""
                check : %s.bak (new stats), %s.old (backup of old stats) %s
                """% (save)* 3)
            raise( e )



    def stats(self):
        """Prints a nicely formatted list of statistics about the package"""

        self.downloads # explicitly call, so we have first/last upload data

        params = (
            self.package_name,
            self.first_upload,
            self.first_upload_rel,
            self.last_upload,
            self.last_upload_rel,
            len(self.releases),
            self.max(),
            self.min(),
            self.average(),
            self.total(),
        )

 
        if not self.quiet:
            print( """PyPI Package statistics for: %s

    First Upload: %40s (%r)
    Last Upload:  %40s (%r)
    Number of releases: %34s
    Most downloads:    %35s
    Fewest downloads:  %35s
    Average downloads: %35s
    Total downloads:   %35s
""" % params)
        self.save()


"""sur recommandation de grenoya :)"""
parser = argparse.ArgumentParser(description = '''
    Gathers download stats from pypi regarding the download information 
    of the geiven package.  Print them, and stores them in a file for further
    use. If no package name are provided, it will try to get all packages
    known from previously fetched stats.

    -q will print less output (only warnings and errors)
    ''')



    

parser.add_argument("-q", "--quiet",
    action="store_true",
    help="less verbose output",
    default=False,
    )
parser.add_argument('_package', metavar='package_name' , 
    nargs='*', help='package name to be retrieved')


options = parser.parse_args()

guessed = set()
if not options._package:
    saved=path.join(environ["HOME"], ".pipy.stat.json"  )
    try:
        with open(saved  ) as f:
            guessed =  reduce( 
                set.union, 
                    map( 
                        lambda x : set( [  x['name'] ]  ), load(f)
                    )
                )
            print(  "found %s" % ",".join(guessed) )
    except:
        print( "could not load previously stored stats to guess packages" )

for pkg in options._package or guessed :
    locale.setlocale(locale.LC_ALL, '')
    from time import sleep
    try:
        PyPIDownloadAggregator(pkg.strip(),True, options.quiet).stats()
    except Exception as e:
        print( "fetching stats for *%s* failed (%r)" % ( pkg, e) )


   




#!/usr/bin/env python
# -*- coding: "utf-8" -*-

#from distutils.command.build_py import build_py as _build_py
from setuptools import setup
#from distutils.core import setup
#from distutil2 import setup
import sys


def func_test():
    from os import path, environ,unlink
    import shutil
    import subprocess
    from json import load
    to_backup=False
    abort=True
    save= path.join(environ["HOME"] , ".pipy.stat.json")
    try:
        if path.exists(save):
            print("backuping previous saved stat in %s to %s.original" % (
                save,save) )
            shutil.move(save, "%s.original" % save)
            to_backup=True

        test_dl=subprocess.call([ "python","./pypi_get_stat.py",'-q', 'archery' ])
        with open(path.join(environ["HOME"] , ".pipy.stat.json")) as  f:
            res=load(f)
            if any(map(lambda d:d.get("name", "") == 'archery', res)):
                abort=False
            else:
                print(
                "This package does not work as is, please open a ticket %s" % e)
                print("Error")
    except Exception as e:
        abort=True
        print(
        "This package does not work as is, please open a ticket %s" % e)
        print("Error")
    finally:
        if to_backup:
            unlink(save)
            shutil.move("%s.original" %save, save)
            print("restoring original stats in %s.original to %s" % (
                save,save)  )
    return not abort

if "sdist" in sys.argv or "install" in sys.argv or "bdist_egg" in sys.argv:
    if func_test():
        print( "###test are successful")
    else:
        print("arg")
        raise Exception("Test did not passed")

setup(
        name='pypi-stat',
        version='1.3.0',
        author='Julien Tayon',
        author_email='julien@tayon.net',
        url= 'https://github.com/jul/pypi-stat',
        packages=[],
        scripts = [ 'pypi_get_stat.py', 'pypi_graph_stat.py' ],
        license="PYTHON SOFTWARE FOUNDATION LICENSE VERSION 2",
        description='solution for getting packages stat, and graphing them',
        install_requires=[
                "numpy>=1.5.0",
                "matplotlib",
                "argparse",
        ],
        long_description=open("README.rst").read(),
        classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Python Software Foundation License',
          'Programming Language :: Python :: 2.6',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python',
          ],
)
